/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef MPEGFILE_H
#define MPEGFILE_H

// for reading file size
#include <sys/types.h>
#include <sys/stat.h>

// because of definitions in header
#include "header.H"
#include "allLayers.H"
#include "allFormat.H"
#include "mpegPlatform.h"

#ifdef __cplusplus

#include <algorithm>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <string.h>

using namespace std;


//--------------------------------------------------------------------------

// Class to open mpeg files, checking the file type
// and creating audio object of according type (layer I/II/III)
// It inherits all functionality to parse frame headers from
// the Header class.
// It also interfaces to required data of the frame content.
class CSAPI_MPEG MPEGfile : public Header, public AllFormat {

friend class Layer1;
friend class Layer2;
friend class Layer3;

#define MAX_BUFSIZE      1728     // no of bytes for storing a frame
  // mpeg1:
  // LI:   max 448kbit/s / 32kHz * 12 slots = 168 slots => 672 byte
  // LII:  max 384kbit/s / 32kHz * 144 slots = 1728 byte
  // LIII: max 320kbit/s / 32kHz * 144 slots = 1440 byte
  // mpeg2:
  // LI:   max 256kbit/s / 16kHz * 12 slots = 192 slots => 768 byte
  // LII:  max 160kbit/s / 16kHz * 144 slots = 1440 byte
  // LIII: max 160kbit/s / 16kHz * 144 slots = 1440 byte
 
public:
  MPEGfile (string filename);
  ~MPEGfile();

  /* check whether file was opened ok */
  bool file_ok();

  /* info about the file (implements virtual function of AllFormat) */
  bool   is_stereo();
  int    channel();
  double sampling_rate();

  /* time functions (implements virtual function of AllFormat) */
  float sample_duration (Resolution res = LOW);

  /* window functions (implements virtual function of AllFormat) */
  bool         seek_window (long w_nb);
  bool         next_window (Resolution res = LOW);
  bool         skip_window();
  unsigned int timeticks (Resolution res = LOW);

  /* access functions (implements virtual function of AllFormat) */
  double       freq_value (unsigned int ch, unsigned int sb, unsigned int nb, 
			   Resolution res = LOW);
  short        pcm         (unsigned int ch, unsigned int number);
  unsigned int nb_subbands (Resolution res = LOW);

  /* decoder (implements virtual function of AllFormat) */
  long decode (short * ptr, long windows, Channels ch); 

  /* end of data reached? */
  bool data_available();

  /* ----------------------- */
  /* MPEG specific functions */

  /* returns the current granule number */
  int  selected_granule();

  /* returns the current frame number */
  long frame();

  /* skips to 1st granule of next frame no matter what granule/window one's at
     use only if you want to get frame information - 
     avoid its use for analysis matters otherwise */
  bool goTo_nextFrame( Resolution res = LOW);

  /* layer-specific access to audio data */
  int   bitallocation    (unsigned int ch, unsigned int sb);
  int   scfsi            (unsigned int ch, unsigned int sb);
  float scalefactor      (unsigned int ch, unsigned int sb);
  int   sample           (unsigned int ch, unsigned int sb, 
			  unsigned int no);
  double restored_sample (unsigned int ch, unsigned int sb, 
			  unsigned int no);
  short pcm_sample       (unsigned int ch, unsigned int sb, 
			  unsigned int no, unsigned int ss=0);
  
  /* layer-III specific data */
  double mdct_sample     (unsigned int ch, unsigned int ssb);
  
  void printSideinfo();
  unsigned int part2_3_length     (unsigned int ch);
  unsigned int big_values         (unsigned int ch);
  unsigned int count1_values      (unsigned int ch);
  unsigned int global_gain        (unsigned int ch);
  unsigned int scalefac_compress  (unsigned int ch);
  bool         window_switching   (unsigned int ch);
  unsigned int blocktype          (unsigned int ch);
  bool         mixedblock         (unsigned int ch);
  unsigned int table_select       (unsigned int ch, unsigned int area);
  unsigned int subblock_gain      (unsigned int ch, unsigned int area);
  unsigned int region0_samps      (unsigned int ch);
  unsigned int region1_samps      (unsigned int ch);
  unsigned int region2_samps      (unsigned int ch);
  unsigned int preflag            (unsigned int ch);
  unsigned int scalefac_scale     (unsigned int ch);
  unsigned int count1table_select (unsigned int ch);
  unsigned int scf_band_bound_l   (unsigned int subbandindex);
  unsigned int scf_band_bound_s   (unsigned int subbandindex);
  unsigned int slen1              (unsigned int ch);
  unsigned int slen2              (unsigned int ch);
  
protected:
  // audio data stored in bitbuffer
  unsigned int buffer[MAX_BUFSIZE/4];

  unsigned int bitsread; // from buffer
  unsigned int readbitsfrombuffer(unsigned int nobits);

private:

  AllLayers *audio;
  
  //pointer to the sound file
  FILE *fd;
    
  // remembers which layer the last frame belonged to
  Layer lastlayer;

  // store the current granule analysed
  int gr_current;

  // true if the current frame is analysed
  bool analysed;

  // DecodedLevel used to analysed the current frame if already done
  DecodeLevel decoded;

  // frame number
  long frameNo;

  // private functions
  bool create_layer();
  bool read_checksum();
  bool buffer_data();
  int  nb_granules();
  bool parse_header();
  bool parse_data(DecodeLevel decode = dec_pcm);
  bool parse_frame(DecodeLevel decode = dec_pcm); // header+data
  bool skip_data();
  bool skip_frame();  // header+data

};

//--------------------------------------------------------------------------

#else /* __cplusplus */

typedef struct MPEGfile MPEGfile;

#endif /* __cplusplus */


#endif
