/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef MAAATESOUNDFILE_H
#define MAAATESOUNDFILE_H

// for reading file size
#include <sys/types.h>
#include <sys/stat.h>

#include "allFormat.H"
#include "tier1Platform.h"

// type of the file to be analysed (encoding format)
/*----- add here the new filetypes ----*/
typedef enum { NOFILE = 0 , MPEG } Format;


#ifdef __cplusplus

#include <algorithm>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <string.h>
#include <math.h>


using namespace std;


//--------------------------------------------------------------------------

class CSAPI_TIER1 SOUNDfile {

public:

  /* constructors and destructor */
  SOUNDfile () : format(NULL),filetype(NOFILE),maxim(0.0),which_max(-1) {}
  SOUNDfile (string filestr);
  virtual ~SOUNDfile ();


  /* about this file */
  string file();         //give file name
  bool   file_ok();       //return whether file was opened ok
  Format file_type();    //give file type (MPEG,...)
  bool is_stereo();      //true if the file is a stereo file
  int channels();        //give the number of channels
  double sampling_rate(); //give the pcm sampling rate in KHz

  /* time functions */
  float at_time();                               //actual time position in sec
  float file_duration();                         //file duration in sec
  float sample_duration( Resolution res = LOW ); //sample duration in sec = time resolution
  bool seek_time( float t );                     //go to time position t in sec
  float window_duration();                       //window duration in sec

  /* window functions */
  long at_window();                               //actual window number
  long file_window_number();                      //total number of windows in the file
  unsigned int timeticks( Resolution res = LOW ); // number of samples in one window
  bool seek_window( long w_nb );                  //go to window nb w_nb

  /* matching functions */
  long time2window( float t );    // convert second into window number
  float window2time( long w_nb ); // convert window number into second

  /* extract and skip */
  bool next_window(Resolution res = LOW); //go to next window and analyse it with res
  bool skip_window(); //go to next window without analysing it
  bool data_available();


  /* access function */

  //subband samples of the current window
  double freq_value( unsigned int ch, unsigned int sb, unsigned int nb = 0, 
		     Resolution res = LOW );

  // return the pcm value number no of a channel ch
  short pcm (unsigned int ch, unsigned int no);


  // number of subbands for that res
  unsigned int nb_subbands( Resolution res = LOW );

  
  /* Analysis functions */

  //time resolution = 1 subband sample
  
  //give the mean of samples nb in subband sb over channels if available
  double freqvalue_st_mean( unsigned int sb, unsigned int nb = 0, Resolution res = LOW); 
  
  //give the rms of samples nb in subband sb over channels if available
  double freqvalue_st_rms( unsigned int sb, unsigned int nb = 0, Resolution res = LOW);
  
  //give the normalised subband energy samples
  double normalised_sb_nrj( unsigned int sb, unsigned int nb = 0, Resolution res = LOW);
  
  //time resolution = 1 window

  //give a coarse idea of the value in that subband
  double subband_scalefactor ( unsigned int sb, Resolution res = LOW);

  //give the mean of samples over the current window
  double subband_mean ( unsigned int sb, Resolution res = LOW);

  //give the rms of samples over the current window
  double subband_rms ( unsigned int sb, Resolution res = LOW);

  //give the normalised subband energy over the current window  
  double subband_nrj ( unsigned int sb, Resolution res = LOW);


  /* decoder */

  //fill buffer with pcm samples corresponding to the "windows" 
  //windows following the current window.
  //This moves the current position pointer while decoding
  //and return the number of samples in buffer.
  long decode (short * buffer, long windows, Channels ch); 


  /* File format specific pointer */

  //pointer to an instance of AllFormat
  AllFormat *format;


private:

  //type of the file
  Format filetype;    

  //store the maximum over all subbands
  double maxim;

  //used by normalised_sb_nrj and subband_nrj
  //tell what maxim means:
  //  -1: meaningless,
  // >=0: give the subband samples number to which it is related,
  //  -2: calculated over all the window
  int which_max;

};

//--------------------------------------------------------------------------

extern "C" {

#else /* __cplusplus */

typedef struct SOUNDfile SOUNDfile;

#endif /* __cplusplus */


/* constructors and destructor */
  CSAPI_TIER1 SOUNDfile * maaateP_new_soundfile(char * filestr); // create a new SOUNDfile
  CSAPI_TIER1 void maaateP_destroy(SOUNDfile * sf);             // delete sf


/* about this file */
  CSAPI_TIER1 char * maaateP_get_filename(SOUNDfile * sf); //file name
  CSAPI_TIER1 Format maaateP_get_filetype(SOUNDfile * sf); //file type (MPEG,...)
  CSAPI_TIER1 int   maaateP_is_stereo(SOUNDfile * sf);     //tell if it is a stereo file
  CSAPI_TIER1 int maaateP_channels(SOUNDfile * sf);        //number of channels
  CSAPI_TIER1 double maaateP_samplingrate(SOUNDfile * sf); //pcm sampling rate in KHz

/* time functions */
  CSAPI_TIER1 float maaateP_at_time(SOUNDfile * sf);                         //current time position in sec
  CSAPI_TIER1 float maaateP_file_duration(SOUNDfile * sf);                   //file duration in sec
  CSAPI_TIER1 float maaateP_sample_duration(SOUNDfile * sf, Resolution res); //sample duration in sec = time resolution
  CSAPI_TIER1 int   maaateP_seek_time(SOUNDfile * sf, float t );             //go to time position t in sec
  CSAPI_TIER1 float maaateP_window_duration(SOUNDfile * sf);                 //window duration in sec


/* window functions */
  CSAPI_TIER1 long  maaateP_at_window(SOUNDfile * sf);                        //current window number
  CSAPI_TIER1 long  maaateP_file_window_number(SOUNDfile * sf);               //total number of windows in the file
  CSAPI_TIER1 unsigned int maaateP_timeticks(SOUNDfile * sf, Resolution res); // number of samples in one window
  CSAPI_TIER1 int   maaateP_seek_window(SOUNDfile * sf, long w_nb );          //go to window nb w_nb

/* matching functions */
  CSAPI_TIER1 long  maaateP_time2window(SOUNDfile * sf, float t );   //convert second into window number
  CSAPI_TIER1 float maaateP_window2time(SOUNDfile * sf, long w_nb ); //convert window number into time


/* extract and skip */
  CSAPI_TIER1 int   maaateP_next_window(SOUNDfile * sf, Resolution res); //go to next window and analyse it with resolution res
  CSAPI_TIER1 int   maaateP_skip_window(SOUNDfile * sf);                 //go to next window whitout analysing it

/* access function */
  CSAPI_TIER1 double maaateP_freq_value(SOUNDfile * sf, unsigned int ch, 
			  unsigned int sb, unsigned int nb, 
			  Resolution res);                          //subband sample of the current window
  CSAPI_TIER1 unsigned int maaateP_nb_subbands(SOUNDfile * sf, Resolution res);  //number of subbands for that res

/* Analyse functions */

  //time resolution = 1 subband sample
  
  //give the mean of samples nb in subband sb over channels if available
  CSAPI_TIER1 double maaateP_freqvalue_st_mean(SOUNDfile * sf, unsigned int sb, 
				   unsigned int nb, Resolution res);  

  //give the rms of samples nb in subband sb over channels if available
  CSAPI_TIER1 double maaateP_freqvalue_st_rms(SOUNDfile * sf, unsigned int sb, 
				unsigned int nb, Resolution res);

  //give the normalised subband energy samples
  CSAPI_TIER1 double maaateP_normalised_sb_nrj(SOUNDfile * sf, unsigned int sb, 
				 unsigned int nb, Resolution res);

  //time resolution = 1 window

  //give the mean of samples over the current window
  CSAPI_TIER1 double maaateP_subband_mean (SOUNDfile * sf, unsigned int sb, Resolution res);

  //give the rms of samples over the current window
  CSAPI_TIER1 double maaateP_subband_rms (SOUNDfile * sf, unsigned int sb, Resolution res);

  //give the normalised subband energy over the current window  
  CSAPI_TIER1 double maaateP_subband_nrj ( unsigned int sb, Resolution res);

/* decode */

  //fill buffer with pcm samples corresponding to the "windows" 
  //windows following the current window.
  //This moves the current position pointer while decoding
  //and return the number of samples in buffer.
  CSAPI_TIER1 long maaateP_decode (SOUNDfile * sf, short * buffer, long windows, Channels ch); 

/* File format specific pointer access */
  CSAPI_TIER1 AllFormat * maaateP_format_access(SOUNDfile * sf);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
