/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef ALLFORMAT_H
#define ALLFORMAT_H

// channels used by decode to return pcm samples
typedef enum { LEFT=0, RIGHT, STEREO } Channels;

// frequency resolution
typedef enum { NO = 0, LOW, HIGH, PCM } Resolution;

#include "tier1Platform.h"

#ifdef __cplusplus

#include <algorithm>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <string>

// for reading file size
#include <sys/types.h>
#include <sys/stat.h>

using namespace std;

#ifndef PI
#define PI            3.14159265358979
#endif

//--------------------------------------------------------------------------

// abstract class to define the `format` to be followed by each format, 
//a kind of normalisation!
class CSAPI_TIER1 AllFormat {
public:

  /* virtual destructor */
  //  virtual ~AllFormat() = 0;

  /* information about the file to be analysed */
  virtual bool   is_stereo() = 0;
  virtual double sampling_rate() = 0;
  virtual int    channel() = 0;

  /* number of frequency subbands related to the analysis at the given
     frequency analysis resolution */
  virtual unsigned int nb_subbands( Resolution res = LOW ) = 0;

  /* returns the number of frequency values in an analysis window
     depending on the given frequency analysis resolution */
  virtual unsigned int timeticks (Resolution res=LOW) = 0;

  /* returns the duration of the time interval (in sec) which is related
     to one frequency value at the given frequency analysis resolution */
  virtual float sample_duration (Resolution res=LOW) = 0;

  /* go to the analysis window number w_nb counting from the start of the
     file without analysing the window */
  virtual bool seek_window (long w_nb) = 0;

  /* skip to next window without analysing current window */
  virtual bool skip_window() = 0;

  /* go to the next analysis window while analysing the current window */
  virtual bool next_window (Resolution res=LOW) = 0;

  /* checks whether there is any data available in the analysis file */
  virtual bool data_available() = 0;

  /* returns the frequency value number nb from the given channel, subband
     at the given frequency analysis resolution */
  virtual double freq_value (unsigned int ch, unsigned int sb, 
			     unsigned int nb, Resolution res=LOW) = 0;

  /* return the pcm value number no of a channel ch*/
  virtual short pcm (unsigned int ch, unsigned int no) = 0;

  /* decodes the frequency values of the given channel and  number of windows
     into PCM values to be stored in the given buffer */
  virtual long decode (short * buffer, long windows, Channels ch) = 0; 

  /* accessor to private fields */
  inline string get_filename();
  virtual bool  file_ok() = 0;
  inline long   get_fileDuration();
  inline long   get_windowNo();
  inline float  get_windowDuration();

protected:

  //must be used by all inherited class in the constructor
  inline bool common_constr(string filestr);

  //must be used by all inherited class in the destructor
  inline void common_destr();

  //name of the file
  string filename;
    
  // remember the file duration in number of windows
  long fileDuration;

  // remember the current window number
  long windowNo;
  
  //window duration in second
  float windowDuration;

};

//--------------------------------------------------------------------------


inline string CSAPI_TIER1
AllFormat::get_filename() {
  return filename;
}

inline long CSAPI_TIER1
AllFormat::get_fileDuration() {
  return fileDuration;
}

inline long CSAPI_TIER1
AllFormat::get_windowNo() {
  return windowNo;
}

inline float CSAPI_TIER1
AllFormat::get_windowDuration() {
  return windowDuration;
}

inline bool CSAPI_TIER1
AllFormat::common_constr (string filestr) {
  //allFormat inherited attributs
  filename     = filestr;
  windowNo     = 0;
  fileDuration = -1;
  windowDuration = 0.0;
  return true;
}

inline void CSAPI_TIER1
AllFormat::common_destr () {
  return;
}

#else /* __cplusplus */

typedef struct Allformat AllFormat;

#endif /* __cplusplus */

#endif
