/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef LAYER3_H
#define LAYER3_H

#include "allLayers.H"
#include "MPEGfile.H"
#include "mpegPlatform.h"

using namespace std;

//--------------------------------------------------------------------------

// class definition
class CSAPI_MPEG Layer3 : public AllLayers {

  // Layer 3 specific header
  struct Sideinfo {
    unsigned int main_data_begin;  // 9 bit
    unsigned int private_bits;     // 3/5 bit
    struct {
      unsigned scfsi[4];         // 1 bit
      struct gr_info_s {
	unsigned part2_3_length;         // 12 bit
	unsigned big_values;             // 9 bit
	unsigned global_gain;            // 8 bit
	unsigned scalefac_compress;      // 4 bit
	unsigned window_switching_flag;  // 1 bit
	unsigned block_type;             // 2 bit
	unsigned mixed_block_flag;       // 1 bit
	unsigned table_select[3];        // 5 bit
	unsigned subblock_gain[3];       // 3 bit
	unsigned region0_count;          // 4 bit
	unsigned region1_count;          // 3 bit
	unsigned preflag;                // 1 bit
	unsigned scalefac_scale;         // 1 bit
	unsigned count1table_select;     // 1 bit
      } gr[2]; // for each granule
    } ch[2]; // for each channel
  };
 
  // structure to store scalefactors in
  struct ScaleFacTable {
    int l[22];            // sfb (long windows)
    int s[3][13];         // window*sfb (short windows)
  };
 
public:
    Layer3(MPEGfile *mp);
    
    int    bitallocation (unsigned int ch, unsigned int sb) {
      /* Layer 3 does not have any bitallocation information */
      return -1;
    };
    int    scfsi (unsigned int ch, unsigned int scfclass);
           float  scalefactor (unsigned int ch, unsigned int ss, 
			       unsigned int gr);
    int    sample (unsigned int ch, unsigned int sb, 
		   unsigned int ss, unsigned int gr);
    double restored_sample (unsigned int ch,  unsigned int sb,
			    unsigned int ss, unsigned int gr);
    short  pcm_sample (unsigned int ch, unsigned int gr, 
			      unsigned int sb, unsigned int ss);

    // layer-3 specific access function
    double mdct_sample (unsigned int ch, unsigned int gr, 
			       unsigned int ssb);
    // more audio data information
    unsigned int part2_3_length(unsigned int ch, unsigned int gr);
    unsigned int big_values (unsigned int ch, unsigned int gr);
    unsigned int count1_values (unsigned int ch, unsigned int gr);
    unsigned int global_gain (unsigned int ch, unsigned int gr);
    unsigned int scalefac_compress (unsigned int ch, unsigned int gr);
    bool         window_switching (unsigned int ch, unsigned int gr);
    unsigned int blocktype (unsigned int ch, unsigned int gr);
    const char*  blocktype_str (unsigned int ch, unsigned int gr);
    bool         mixedblock (unsigned int ch, unsigned int gr);
    unsigned int table_select(unsigned int ch, unsigned int gr, 
				     unsigned int area);
    unsigned int subblock_gain(unsigned int ch, unsigned int gr, 
				      unsigned int area);
    unsigned int region0_samps (unsigned int ch, unsigned int gr);
    unsigned int region1_samps (unsigned int ch, unsigned int gr);
    unsigned int region2_samps (unsigned int ch, unsigned int gr);
    unsigned int preflag (unsigned int ch, unsigned int gr);
    unsigned int scalefac_scale (unsigned int ch, unsigned int gr);
    unsigned int count1table_select(unsigned int ch, unsigned int gr);
    unsigned int scf_band_bound_l (unsigned int sbindex);
    unsigned int scf_band_bound_s(unsigned int sbindex);
    unsigned int slen1 (unsigned int ch, unsigned int gr);
    unsigned int slen2 (unsigned int ch, unsigned int gr);
    unsigned int granules();

    unsigned int calc_CRC();
    bool parse_data(DecodeLevel decode);


    void printSideinfo();

    bool clearinterbuffer();

private:
    MPEGfile *frame;

#ifdef DEBUG
    long noReadBits;
#endif

    // audio data stored in intermediate bitbuffer.
    // This is a "circular buffer" and takes any spare bytes from
    // the frame's buffer to store between indexes bufstart and bufend.
#define MAX_INTER_BUFSIZE 1920 // max no of bytes stored (7680 bit/granule)
    unsigned char inter_buffer[MAX_INTER_BUFSIZE];
    unsigned int bufstart, bufend;
    unsigned int whichbit;
    unsigned int main_data_bitsread;
    unsigned int endSpectralBits;   // to remember how many bits to read
    unsigned int spectral_bitsread; // to calculate count1 values to read
    unsigned int readbits(unsigned int nobits);
    bool         setbufstart();
    bool         savetointerbuffer();

    // read huffman decoder table
    bool read_decoder_table();

    // remember how many count1-values were processed
    // granules()*channels() count1 values
    int count1_vals[2][2];

    // AUDIO DATA CONTENT:
    // 9, 17 or 32 byte to be read into sideinfo header
    Sideinfo si;
    // 0-4 bit read per entry
    // granules()*channels() scalefactors
    ScaleFacTable scalefac[2][2];
    // quantized value for frequency lines (0-19 bit to be read per entry)
    // granules()*channels()*subbands()(32)*scalefactorbands()(18)
    long int is[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() restored samples
    double xr[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() restored stereo
    double lr[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() reordered samples
    double ro[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*subbands()*scalefactorbands() hybrid filter input
    double hybridIn[2][2][SBLIMIT][SSLIMIT];
    // granules()*channels()*scalefactorbands()*subbands() hybrid filter output
    double hybridOut[2][2][SSLIMIT][SBLIMIT];
    // granules()*channels()*subsubbands()*subbands() pcm samples
    short pcm[2][2][SSLIMIT][SBLIMIT];

    // frame processing functions
    void decode_sideinfo();
    void decode_scale_I(int gr, int ch);
    void decode_scale_II(int gr, int ch); // for MPEG2
    void decode_huffmanbits(int gr, int ch); // sample data
    bool huffman_decoder(struct huffcodetab *h, // huffman code record
			 unsigned int *x,       // decoded x value
			 unsigned int *y,       // decoded y value
			 unsigned int *v,       // decoded v value
			 unsigned int *w);
    void restore_samples(int gr, int ch);
    void decode_jstereo(int gr);
    void i_stereo_k_values(int ch, int gr, char is_pos, int i, 
			   double k[2][SBSSLIMIT]); // MPEG2
    void reorder(int gr, int ch);
    void antialias(int gr, int ch);
    void hybrid(int gr, int ch);
    void inv_mdct(double in[18], double out[36], int bt);
    void polyphase(int ch, int gr);

    // access to class member values
    unsigned int main_data_begin();
    unsigned int private_bits();
    unsigned int region0_count (unsigned int ch, unsigned int gr);
    unsigned int region1_count (unsigned int ch, unsigned int gr);
    unsigned int scfsi_group (unsigned int ch, unsigned int group);
    unsigned int sideinfo_length();
    unsigned int main_data_slots();

    unsigned int scf_bands (unsigned int ch, unsigned int gr);


};

//--------------------------------------------------------------------------

#endif
