/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SEGMENTDATA_H
#define SEGMENTDATA_H

#include "tier2Platform.h"

#ifdef __cplusplus

#include <math.h>
#include <float.h>
#include <stdio.h>
#include <iostream>
#include <vector>

using namespace std;

//class CSAPI_TIER2 SegmentData;
 
//--------------------------------------------------------------------------
 
// class definition
class CSAPI_TIER2 SegmentData {
public:
  // different constructors
  SegmentData(): starttime(0), endtime(0), columns(0), rows(0),
		 id(0), flag(' '), conf(0.0), data(NULL), colFilled(0),
		 sum_save(0.0), min_save(DBL_MAX), max_save(DBL_MIN) {}
  SegmentData(double t1, double t2, int col=0, int row=0, 
	      int i=0, char f=' ', double conf=0.0);
  
  // copy constructor
  SegmentData(const SegmentData& sd);

  SegmentData(const SegmentData& sd,
	      int st_row, int end_row,
	      int i=0, char f=' ', double cnf=0.0);
  
  // destructor
  ~SegmentData();
  
  // comparison between two SegmentData objects with respect to start time
  friend bool operator< (const SegmentData& sd1, const SegmentData& sd2) {
    return (sd1.start() < sd2.start());
  }
  
  // comparison between two SegmentData objects with respect to end time
  friend bool operator> (const SegmentData& sd1, const SegmentData& sd2) {
    return (sd1.end() > sd2.end());
  }

  // overload <<
  friend ostream& operator<< (ostream& os, const SegmentData& sd);
  
  // overload =
  SegmentData& operator= (const SegmentData& sd);
  
  // index operator
  double& operator() (int c, int r) {
    if (c < 0 || c >= columns || r < 0 || r >= rows) {
      cerr << "MaaateA: SegmentData: index out of range" << endl;
      return data[0][0];
    } 
    return data[c][r];
  }
  
  // access class content
  double start()      const { return starttime; }
  double end()        const { return endtime; }
  double duration()   const { return endtime-starttime; }
  double resolution() const { return columns>0? (duration()/columns) : 0.0; }
  double filled()      const { return resolution()*colFilled; }
  double filledUntil() const { return filled() + starttime; }
  int    col_filled()  const { return colFilled; }
  int    no_columns() const { return columns; }
  int    no_rows()    const { return rows; }
  int    ident()      const { return id; }
  char   uflag()      const { return flag; }
  double confidence() const { return conf; }
  
  // convert between time and column number
  int time2col (double time) const;
  
  // operators
  double sum (int start, int end, int startrow, int endrow);
  double smin (double start, double end, int startrow, int endrow);
  double smax (double start, double end, int startrow, int endrow);
  double avg (int start, int end, int startrow, int endrow);
  double sum();
  double smin(double downto = -DBL_MAX);
  double smax(double upto = DBL_MAX);
  double avg();
  bool normalise(double downto = -DBL_MAX, double upto = DBL_MAX);
  bool smooth(int start, int end, int startrow, int endrow, int order);
  bool matOut(char* fileName="default.mat");

  void clear_sum() { sum_save=0.0; }
  void clear_min() { min_save=DBL_MAX; }
  void clear_max() { max_save=-DBL_MAX; }
  void clear_all() { clear_sum(); clear_min(); clear_max(); }
  
  void initData (double val);

  // data container
  double **data;
  int columns; // no of time points
  int rows;    // no of components
  int colFilled; // columns until which data is filled in
 
private:
  double starttime; // starttime
  double endtime;   // endtime
  
  int  id;    // user-defined characteristic
  char flag;  // user-defined flag
  double conf; // confidence value [%]
  
  // summarizing info (store after first calculation)
  double sum_save;
  double min_save;
  double max_save;
  
};

extern "C" {
  
#else /* __cplusplus */
  
  typedef struct SegmentData SegmentData;
  
#endif /* __cplusplus */
  
  CSAPI_TIER2 SegmentData * maaateA_sd_new        (double t1, double t2, int col, int row, 
				       int i, char f, double conf);
  CSAPI_TIER2 SegmentData * maaateA_sd_new_default(void);
  CSAPI_TIER2 void          maaateA_sd_delete     (SegmentData *sd);
  
  CSAPI_TIER2 int           maaateA_sd_starts_before(SegmentData *sd1, SegmentData *sd2);
  CSAPI_TIER2 int           maaateA_sd_ends_after   (SegmentData *sd1, SegmentData *sd2);

  CSAPI_TIER2 void          maaateA_sd_print          (SegmentData * sd);
  CSAPI_TIER2 SegmentData * maaateA_sd_copy           (SegmentData *sd);
  CSAPI_TIER2 SegmentData * maaateA_sd_partial_copy   (SegmentData *sd, int st_row, int end_row);
  CSAPI_TIER2 double        maaateA_sd_element        (SegmentData *sd, int c, int r);
  CSAPI_TIER2 double        maaateA_sd_get_start      (SegmentData * sd);
  CSAPI_TIER2 double        maaateA_sd_get_end        (SegmentData * sd);
  CSAPI_TIER2 double        maaateA_sd_get_duration   (SegmentData * sd);
  CSAPI_TIER2 double        maaateA_sd_get_resolution (SegmentData * sd);
  CSAPI_TIER2 double        maaateA_sd_get_filled     (SegmentData * sd);
  CSAPI_TIER2 double        maaateA_sd_get_filledUntil(SegmentData * sd);
  CSAPI_TIER2 int           maaateA_sd_col_filled     (SegmentData * sd);
  CSAPI_TIER2 int           maaateA_sd_no_columns     (SegmentData * sd);
  CSAPI_TIER2 int           maaateA_sd_no_rows        (SegmentData * sd);
  CSAPI_TIER2 int           maaateA_sd_get_ident      (SegmentData * sd);
  CSAPI_TIER2 char          maaateA_sd_get_uflag      (SegmentData * sd);
  CSAPI_TIER2 double        maaateA_sd_get_confidence (SegmentData * sd);

  CSAPI_TIER2 int    maaateA_sd_time2col (SegmentData * sd, double time);

  CSAPI_TIER2 double maaateA_sd_sum_part (SegmentData * sd, int start, int end, 
			      int startrow, int endrow);
  CSAPI_TIER2 double maaateA_sd_min_part (SegmentData * sd, double start, double end, 
			      int startrow, int endrow);
  CSAPI_TIER2 double maaateA_sd_max_part (SegmentData * sd, double start, double end, 
			      int startrow, int endrow);
  CSAPI_TIER2 double maaateA_sd_avg_part (SegmentData * sd, int start, int end, 
			      int startrow, int endrow);
  CSAPI_TIER2 double maaateA_sd_sum      (SegmentData * sd);
  CSAPI_TIER2 double maaateA_sd_min      (SegmentData * sd, double downto);
  CSAPI_TIER2 double maaateA_sd_max      (SegmentData * sd, double upto);
  CSAPI_TIER2 double maaateA_sd_avg      (SegmentData * sd);
  CSAPI_TIER2 int    maaateA_sd_normalise(SegmentData * sd, double downto, double upto);
  
  CSAPI_TIER2 void maaateA_sd_clear_sum (SegmentData * sd);
  CSAPI_TIER2 void maaateA_sd_clear_min (SegmentData * sd);
  CSAPI_TIER2 void maaateA_sd_clear_max (SegmentData * sd);
  CSAPI_TIER2 void maaateA_sd_clear_all (SegmentData * sd);

  CSAPI_TIER2 void maaateA_sd_initData  (SegmentData * sd, double val);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
